class Types(object):

    """
    All data types to create a column.
    """

    BIGINT = "BIGINT"
    BLOB = "BLOB"
    BOOLEAN = "BOOLEAN"
    CHAR = "CHAR"
    DATE = "DATE"
    DATETIME = "DATETIME"
    DECIMAL = "DECIMAL"
    DOUBLE = "DOUBLE"
    INTEGER = "INTEGER"
    INT = "INT"
    NONE = "NONE"
    NUMERIC = "NUMERIC"
    REAL = "REAL"
    STRING = "STRING"
    TEXT = "TEXT"
    TIME = "TIME"
    TIMESTAMP = "TIMESTAMP"
    VARCHAR = "VARCHAR"
