MODLES = """ 
from aquilify.orm.sqlite3 import Table
from aquilify.orm.fields import CharField

# Create your model here...

# Register Models

# Update the `settings` `DATABASE` Dict[Dict[str, str]] -> {
#     MODELS: List[str, str] = [
#         "models.your_model_cls" # pass the path_to model dotted string inside the list.
#     ]
# }
"""



