# AQUILIFY BASE MIDDLEWARE
# refer @noql -> 5806

# Aquilify base middleware is still under testing, it will be released very soon...

# For more info visit :: http://aquilify.vvfin.in/middlewares/base/#main